/*  Copyright (C) MOXA Inc. All rights reserved.

    This software is distributed under the terms of the
    MOXA License.  See the file COPYING-MOXA for details.
*/

/*
    example.c

    Example program form NPort SSDK.

    2007-04-30	Kevin Chu
		new release
    2008-05-30	Albert Yu
		add IPv6 support
*/

/************************************************************************/
/*	MOXA NPort 'Socket Mode' Port Control Program Example               */
/*	Program:                                                            */
/*	   example.c                                                        */
/*	Purpose:                                                            */
/*	   This program will send '1234567890' string                       */
/*	   out from NPort's RS-232 port, whose port Mode                    */
/*	   should be set as 'TCP Server', and read back any                	*/
/*	   incoming data.                                                   */
/*	Setting:                                                            */
/*	   The target port will be configured as:                           */
/*    no parity,                                                        */
/*    8 data bits,                                                      */
/*    1 stop bit,                                                       */
/*     with software(XON/XOFF) flow control,but no hardware(RTS/CTS).   */
/*     1st port and 9600bps is default, however you may change          */
/*     it in command line.                                              */
/*     The target port Mode should be set as 'TCP Server'              	*/
/*     via [Port] [Modes] from NPort CONSOLE utility.                   */
/*	Syntax:                                                             */
/*	   ./example ServerIP [cmd_port(966) data_port(4001) Baud(9600)]]   */
/*                                                                      */
/*	   For example:                                                     */
/*	      ./example 192.168.1.2 966 4001 115200                         */
/*	   The program will send '1234567890' to port 1 on NPort            */
/*	   via NPort's TCP port 4001 at 115200 bps baud                     */
/*     and read back any data on it.                                    */
/*                                                                      */
/*	Environment:                                                        */
/*	   This program is originally developed under RedHat 7.3 & WinXP.   */
/*	   Your environment may be different, if so, the include file       */
/*	   name and other variable may need to be modified to suit          */
/*	   your environment.                                                */
/************************************************************************/

#include	<stdlib.h>
#include	<stdio.h>
#include	<string.h>
#ifndef	WIN32
	#include	<fcntl.h>
	#include	<sys/errno.h>
	#include	<sys/socket.h>
	#include	<netinet/in.h>
	#include	<netdb.h>
	#include	<arpa/telnet.h>
#endif

#include	"socketmode.h"

main(argc, argv)
int	argc;
char *	argv[];
{
	int	fd;
	int	cmdport, i, baud, dataport;
	struct sockaddr_in	des;
	char	buf[100];

#ifdef WIN32
	WSADATA wsaData;
  
	if (WSAStartup(0x202,&wsaData) == SOCKET_ERROR) {
		printf("WSAStartup failed!\n");		
		exit(0);
	}
#endif

	if ( argc == 1 ) {
		printf("\nNo Server IP specified.\n");
		printf("\nSyntax: # ./%s ServerIP [command_port data_port [Baud]]\n", argv[0]);
		printf("For example: # ./%s 192.168.1.2 966 4001 115200\n\n", argv[0]);
		exit(0);
	}
	memset(&des, 0, sizeof(des));
	printf("Hook up a terminal to the first port to test.\n");
	printf("NPort IP addr = %s \n", argv[1]);

	if ( argc > 2 )
		cmdport = atoi(argv[2]);

	if ( argc > 3 )
		dataport = atoi(argv[3]);

	if ( argc > 4 )
	{
		baud = atoi(argv[4]);
		switch( baud )
		{
			case 50: baud = D_IOCTL_50;	break;
			case 75: baud = D_IOCTL_75; break;				
			case 110: baud = D_IOCTL_110; break;	
			case 134: baud = D_IOCTL_134; break;
			case 150: baud = D_IOCTL_150; break;
			case 300:	baud = D_IOCTL_B300;	break;
			case 600:	baud = D_IOCTL_B600;	break;
			case 1200:	baud = D_IOCTL_B1200;	break;
			case 2400:	baud = D_IOCTL_B2400;	break;
			case 4800:	baud = D_IOCTL_B4800;	break;
			case 7200:	baud = D_IOCTL_B7200;	break;
			case 9600:	baud = D_IOCTL_B9600;	break;
			case 19200:	baud = D_IOCTL_B19200;	break;
			case 38400:	baud = D_IOCTL_B38400;	break;
			case 57600:	baud = D_IOCTL_B57600;	break;
			case 115200:	baud = D_IOCTL_B115200; break;
			case 230400:	baud = D_IOCTL_B230400; break;
			case 460800:	baud = D_IOCTL_B460800; break;
			case 921600:	baud = D_IOCTL_B921600; break;
			default:	printf("invalide baud rate %d \n", baud);
					exit(0);
		}
		printf("\nport settings : %s,n,8,1,no RTS/CTS flow control.\n\n", argv[4]);
	}
	else
	{
		baud = D_IOCTL_B9600;
		printf("\nport settings : 9600,n,8,1,no RTS/CTS flow control.\n\n");
	}
	printf("Press any key to continue...\n");
	i = getchar();

	ssio_init();

	/* if you want to enable ssl function, you have to set 
			ssl_enable to 1 when calling ssio_open()*/
#ifdef	SSL_ON			
	if ( (fd = ssio_open(argv[1], cmdport, dataport, 1)) < 0 )
#else		
	if ( (fd = ssio_open(argv[1], cmdport, dataport, 0)) < 0 )
#endif
	{
		printf("\nPort open fail!\n");
		printf("\nRe-examine the port Mode from NPort CONSOLE.\n");
		exit(0);
	}
	printf("Port opened OK.\n");

	if ( ssio_ioctl(fd,baud,D_IOCTL_BIT8+D_IOCTL_STOP1+D_IOCTL_NONE) < 0 )
	{
		printf("Port ioctl error!\n");
		ssio_close(fd);
		exit(0);
	}
	printf("Port ioctl OK. \n");

	if ( ssio_flowctrl(fd, 0xC) < 0 )
	{
		printf("Port flow control error!\n");
		ssio_close(fd);
		exit(0);
	}
	printf("Port flow control OK.\n");

	if ( ssio_flush(fd, D_FLUSH_ALLBUFFER) < 0 )
	{
		printf("Port flush error!\n");
		ssio_close(fd);
		exit(0);
	}
	printf("Port flush OK.\n");

	i = ssio_lstatus(fd);
	if ( i < 0 )
	{
		printf("Port lstatus error!\n");
		ssio_close(fd);
		exit(0);
	}
	printf("\n");
	if ( i & 1 )
		printf("DSR ON / ");
	else	
		printf("DSR OFF / ");
		
	if ( i & 2 )
		printf("CTS ON / ");
	else	
		printf("CTS OFF / ");	
		
	if ( i & 4 )
		printf("DCD ON / ");
	else
		printf("DCD OFF / ");	
	printf("\n");

	if ( ssio_write(fd, "1234567890", 10) != 10 ) 
	{
		printf("write data [1234567890] fail!\n");
		ssio_close(fd);
		exit(0);
	}
	else
		printf("write data [1234567890] OK.\n");
	printf("Waiting any data from data_port %d .....\n", dataport);

	i = ssio_read(fd, buf, 100);
	if ( i > 0 ) 
	{
		buf[i] = 0;
		printf("Read: [%s]\n", buf);
	} 
	else
		printf("read return %d !!!\n", i);

	i = ssio_breakcnt(fd);
	if (i >= 0)
		printf("break count = %d\n", i);
	else
		printf("get break count fail!");

	printf("send a break message...");
	if (ssio_break(fd, 1000) < 0)
		printf("failed!\n");
	else
		printf("OK.\n");

	i = ssio_breakcnt(fd);
	if (i >= 0)
		printf("break count = %d\n", i);
	else
		printf("get break count fail!");

	ssio_close(fd);
	
#ifdef WIN32
	WSACleanup();
#endif	
}

